//go:build cgo && !disable_codec_libaom

package libaom

import (
	"git.gammaspectra.live/S.O.N.G/Ignite/decoder/y4m"
	"git.gammaspectra.live/S.O.N.G/Ignite/testdata"
	"os"
	"runtime"
	"sync"
	"testing"
)

func TestVersion(t *testing.T) {
	t.Logf("libaom version: %s", Version())
}

func TestEncode(t *testing.T) {
	if testing.Short() {
		t.Skip("skipping test in short mode")
	}
	f, err := os.Open(testdata.Y4M_Sintel_Trailer_720p24_YUV420_8bit)
	if err != nil {
		t.Fatal(err)
	}
	defer f.Close()

	if pipe, err := y4m.NewDecoder(f, nil); err != nil {
		t.Fatal(err)
	} else {
		defer pipe.Close()
		target, err := os.CreateTemp("", "encode_test_*.ivf")
		if err != nil {
			t.Fatal(err)
			return
		}

		defer func() {
			name := target.Name()
			target.Close()
			os.Remove(name)
		}()

		stream := pipe.DecodeStream()

		settings := make(map[string]any)
		settings["threads"] = runtime.NumCPU()
		settings["row-mt"] = 1
		settings["cpu-used"] = 8
		//settings["use-16bit-internal"] = 1
		settings["frame-parallel"] = 1
		settings["lag-in-frames"] = 2
		settings["tile-columns"] = 1
		settings["tile-rows"] = 1

		if encoder, err := NewEncoder(target, stream.Properties(), settings); err != nil {
			t.Fatal(err)
		} else {
			defer encoder.Close()
			s := stream.Copy(2)
			encoded := 0
			var wg sync.WaitGroup
			wg.Add(1)
			go func() {
				defer wg.Done()
				for range s[0].Channel() {
					if encoded%10 == 0 {
						t.Logf("frame %d/1253", encoded)
					}
					encoded++
				}
			}()

			if err = encoder.EncodeStream(s[1]); err != nil {
				t.Fatal(err)
			}

			wg.Wait()

			if encoded != 1253 {
				t.Fatalf("expected %d frames, got %d", 1253, encoded)
			}
		}
	}
}
